import java.util.Random;

public class FactoryMapUtil {

    /**
     * Creates the DemoMap from the Exercise description.
     *
     * @return
     */
    public static FactoryMap createDemoMap() {
        FactoryMap.Obstacle obst1 = new FactoryMap.Obstacle(10, 13, 4, 12);
        FactoryMap.Obstacle obst2 = new FactoryMap.Obstacle(3, 0, 3, 20);
        FactoryMap.Obstacle obst3 = new FactoryMap.Obstacle(17, 5, 6, 20);
        return new FactoryMap(25, 25, new FactoryMap.Obstacle[]{obst1, obst2, obst3});
    }

    public static FactoryMap.Obstacle createRandomObstacle(Random rand, int sizeX, int sizeY) {
        int posX, posY, dx, dy;
        do {
            posX = rand.nextInt(sizeX);
            posY = rand.nextInt(sizeY);

            dx = (sizeX - posX - 1) > 0 ? rand.nextInt(sizeX - posX - 1) : 0;
            dy = (sizeY - posY - 1) > 0 ? rand.nextInt(sizeY - posY - 1) : 0;
        } while ((posX == 0 && posY == 0) || (posX + dx == sizeX - 1 && posY + dy == sizeY - 1));
        return new FactoryMap.Obstacle(posX, posY, dx, dy);
    }

    public static FactoryMap.Obstacle createRandomBorder(Random rand, int sizeX, int sizeY) {
        int posX, posY, dx, dy;
        boolean coin;
        do {
            coin = rand.nextBoolean();
            if (coin) {
                coin = rand.nextBoolean();
                if (coin) {
                    posX = 0;
                    dx = (sizeX - posX - 2) > 0 ? rand.nextInt(sizeX - posX - 2) : 0;
                } else {
                    posX = rand.nextInt(sizeX);
                    dx = sizeX - posX;
                }
                posY = rand.nextInt(sizeY);
                dy = (sizeY - posY - 1) > 0 ? rand.nextInt(sizeY - posY - 1) : 0;
            } else {
                coin = rand.nextBoolean();
                if (coin) {
                    posY = 0;
                    dy = (sizeY - posY - 2) > 0 ? rand.nextInt(sizeY - posY - 2) : 0;
                } else {
                    posY = rand.nextInt(sizeY);
                    dy = sizeY - posY;

                }
                posX = rand.nextInt(sizeX);
                dx = (sizeX - posX - 1) > 0 ? rand.nextInt(sizeX - posX - 1) : 0;
            }
        } while ((posX == 0 && posY == 0) || (posX + dx == sizeX - 1 && posY + dy == sizeY - 1));
        return new FactoryMap.Obstacle(posX, posY, dx, dy);
    }

    /**
     * Creates a Random FactoryMap
     *
     * @param sizeX        the height of the map
     * @param sizeY        the width of the map
     * @param numObstacles the number of random obstacles
     * @param seed         the random number generator seed
     * @return a random map according to the input parameters
     */
    public static FactoryMap createRandomMap(int sizeX, int sizeY, int numObstacles, int seed) {
        Random rand = new Random(seed);
        FactoryMap.Obstacle[] obstacles = new FactoryMap.Obstacle[numObstacles];
        for (int ind = 0; ind < obstacles.length; ind++) {
            obstacles[ind] = createRandomObstacle(rand, sizeX, sizeY);
        }
        return new FactoryMap(sizeX, sizeY, obstacles);
    }
}
